;----------------------------------------------------------;
; BOS 0.03                       Christoffer Bubach, 2004. ;
;----------------------------------------------------------;
;                                                          ;
;     Keyboard functions. IRQ, INTs and more.             ;
;                                                          ;
;----------------------------------------------------------;


;--------------;
;  Variabels   ;
;--------------;
	  kbd_raw_buffer  db   0		      ; raw scancode buffer

     ;-------------------------------------------------------------------------;
     ; _________________        Flag byte:                                     ;
     ; |1|1|0|0|0|1|1|1|                                                       ;
     ; +---------------+                     1 = True   0 = False              ;
     ;  | | | | | | | +--->  shift key                                         ;
     ;  | | | | | | +----->  ctrl key                                          ;
     ;  | | | | | +------->  alt key                                           ;
     ;  | | | | +--------->  reserved bit                                      ;
     ;  | | | +----------->  reserved bit                                      ;
     ;  | | +------------->  reserved bit                                      ;
     ;  | +--------------->  ctrl + alt + del                                  ;
     ;  +----------------->  key released                                      ;
     ;-------------------------------------------------------------------------;
	  kbd_flags	  db   0		      ; flag byte

     ;-------------------------------------------------------------------------;
     ; _________________        LED status byte:                               ;
     ; |0|0|0|0|0|1|1|1|                                                       ;
     ; +---------------+                     1 = True   0 = False              ;
     ;            | | +--->  scroll lock                                       ;
     ;            | +----->  num lock                                          ;
     ;            +------->  caps lock                                         ;
     ;-------------------------------------------------------------------------;
	  kbd_status	  db   0		      ; LED statusbyte


;----------------------;
;   Keyboard IRQ       ;
;----------------------;
keyboard_isr:
	  push	  eax

     ;-------------------------;
     ;   get the scancode      ;
     ;-------------------------;
	  xor	  eax, eax
	  in	  al, 0x60

     ;------------------------------;
     ;  check if key was released   ;
     ;------------------------------;
	  test	  al, 0x80
	  jz	  .key_down
	  and	  byte [kbd_flags], 01111111b	      ; key up..

	  cmp	  al, 42+128			      ; left shift up?
	  je	  .shift_up
	  cmp	  al, 54+128			      ; right shift up?
	  je	  .shift_up

	  cmp	  al, 29+128			      ; ctrl up?
	  je	  .ctrl_up

	  cmp	  al, 83+128			      ; del up?
	  je	  .del_up

	  cmp	  al, 56+128			      ; alt up?
	  je	  .alt_up
	  jmp	  .end

     ;--------------------;
     ;  it was released   ;
     ;--------------------;
     .shift_up:
	  and	  byte [kbd_flags], 11111110b
	  jmp	  .end

     .del_up:
	  jmp	  .CAD_off

     .ctrl_up:
	  and	  byte [kbd_flags], 11111101b	   ; ctrl off.
	  jmp	  .CAD_off

     .alt_up:
	  and	  byte [kbd_flags], 11111011b	   ; alt off.
	  jmp	  .CAD_off

     .CAD_off:
	  test	  byte [kbd_flags], 01000000b
	  jz	  .CAD_is_off
	  and	  byte [kbd_flags], 10111111b	   ; ctrl+alt+del bit off.
     .CAD_is_off:
	  jmp	  .end

     ;----------------------------------------------;
     ;  a key was pressed, check for special keys   ;
     ;----------------------------------------------;
     .key_down:
	  or	  byte [kbd_flags], 10000000b

     .shift:
	  cmp	  al, 42
	  jnz	  .check_rshift
	  or	  byte [kbd_flags], 00000001b
	  jmp	  .end

     .check_rshift:
	  cmp	  al, 54
	  jnz	  .check_ctrl
	  or	  byte [kbd_flags], 00000001b
	  jmp	  .end

     .check_ctrl:
	  cmp	  al, 29
	  jnz	  .check_alt
	  or	  byte [kbd_flags], 00000010b
	  jmp	  .end

     .check_alt:
	  cmp	  al, 56
	  jnz	  .ctrl_alt_del
	  or	  byte [kbd_flags], 00000100b
	  jmp	  .end

     .ctrl_alt_del:
	  test	  byte [kbd_flags], 00000110b	      ; check for ctrl+alt
	  jz	  .check_caps
	  cmp	  al, 83			      ; check for delete
	  jne	  .check_caps
	  or	  byte [kbd_flags], 01000000b	      ; ctrl+alt+del bit on.

     ;-------------------------------------;
     ;  toggle caps, num and scroll lock   ;
     ;-------------------------------------;
     .check_caps:
	  cmp	  al, 58
	  jnz	  .check_num
	  xor	  byte [kbd_status], 4
	  call	  update_leds
	  jmp	  .end

     .check_num:
	  cmp	  al, 69
	  jnz	  .check_scroll
	  xor	  byte [kbd_status], 2
	  call	  update_leds
	  jmp	  .end

     .check_scroll:
	  cmp	  al, 70
	  jnz	  .end
	  xor	  byte [kbd_status], 1
	  call	  update_leds
	  jmp	  .end

     ;-----------------------------------;
     ;  put the scancode in the buffer   ;
     ;-----------------------------------;
     .end:
	  mov	  [kbd_raw_buffer], al

	  mov	  al, 0x20
	  out	  0x20, al
	  pop	  eax
	  ret



;------------------------------;
;  Update the keyboard LEDs   ;
;------------------------------;
update_leds:
	  push	  ax

	  call	  kbd_wait

	  mov	  al, 0xED
	  out	  0x60, al

	  call	  kbd_wait

	  mov	  al, [kbd_status]
	  out	  0x60, al

	  call	  kbd_wait

	  pop	  ax
	  ret



;------------------;
;  keyboard wait   ;
;------------------;
kbd_wait:
	  jmp	  $+2
	  in	  al, 0x64
	  test	  al, 1
	  jz	  .ok
	  jmp	  $+2
	  in	  al, 0x60
	  jmp	  kbd_wait
     .ok:
	  test	  al, 2
	  jnz	  kbd_wait
	  ret



;-------------------------------------------------------;
;  BOS INT to get a char.                               ;
;          OUT: ah = scan code,      al = ascii         ;
;               bh = flag-byte,      bl = led-byte      ;
;-------------------------------------------------------;
getc:
	  mov	  ax, 0xFF			      ; set buffer to
	  mov	  [kbd_raw_buffer], al		      ; strange value..
     .l1:
	  mov	  al, [kbd_raw_buffer]		      ; loop until its not
	  cmp	  al, 0xFF			      ; strange anymore.. :-)
	  je	  .l1

	  mov	  ah, 0
	  movzx   esi, ax

     ;------------------;
     ;  some checks..   ;
     ;------------------;
	  cmp	  al, 0xFA
	  je	  getc
	  cmp	  al, 0xE0
	  je	  getc
	  test	  al, 0x80
	  jnz	  getc

     ;--------------------------------;
     ;  check for caps, shift & alt   ;
     ;--------------------------------;
	  test	  [kbd_status], 00000100b
	  jnz	  .caps
	  test	  [kbd_flags], 00000001b
	  jnz	  .shift

     ;------------------;
     ;  normal keymap   ;
     ;------------------;
	  mov	  al, [esi+keymap]
	  jmp	  .end

     ;--------------------;
     ;  capslock keymap   ;
     ;--------------------;
     .caps:
	  test	  [kbd_flags], 00000001b
	  jnz	  .caps_and_shift

	  mov	  al, [esi+keymap_caps]
	  jmp	  .end

     ;--------------------------;
     ;  caps and shift keymap   ;
     ;--------------------------;
     .caps_and_shift:
	  mov	  al, [esi+keymap_caps_shift]
	  jmp	  .end

     ;-----------------;
     ;  shift keymap   ;
     ;-----------------;
     .shift:
	  mov	  al, [esi+keymap_shift]
	  jmp	  .end

     ;---------------------------;
     ;  set registers and exit   ;
     ;---------------------------;
     .end:
	  mov	  ah, [kbd_raw_buffer]
	  mov	  bl, [kbd_status]
	  mov	  bh, [kbd_flags]

	  ret