;----------------------------------------------------------;
; BOS 0.03                       Christoffer Bubach, 2004. ;
;----------------------------------------------------------;
;                                                          ;
;     PIC/IRQ handling.                                    ;
;                                                          ;
;----------------------------------------------------------;


;-------------------------------------------;
;  remap PICs to:  CL = pic1  CH = pic2     ;
;-------------------------------------------;
remap_pics:
	  push	  eax
	  push	  edx
	  push	  ebx

     ;---------;
     ;  IWC1   ;
     ;---------;
	  mov	  al, 0x11
	  out	  0x20, al

	  out	  0xA0, al

     ;---------;
     ;  IWC2   ;
     ;---------;
	  mov	  al, cl
	  out	  0x21, al

	  mov	  al, ch
	  out	  0xA1, al

     ;---------;
     ;  IWC3   ;
     ;---------;
	  mov	  al, 0x04
	  out	  0x21, al

	  mov	  al, 0x02
	  out	  0xA1, al

     ;---------;
     ;  IWC4   ;
     ;---------;
	  mov	  al, 0x01
	  out	  0x21, al

	  out	  0xA1, al

	  pop	  ebx
	  pop	  edx
	  pop	  eax
	  ret


;----------------------;
;  disable all IRQs.   ;
;----------------------;
disable_irqs:
	  push	  eax
	  push	  edx

	  mov	  al, 0xFF
	  out	  0x21, al

	  out	  0xA1, al

	  pop	  edx
	  pop	  eax
	  ret


;---------------------;
;  enable all IRQs.   ;
;---------------------;
enable_irqs:
	  push	  eax
	  push	  edx

	  mov	  al, 0x00
	  out	  0x21, al

	  out	  0xA1, al

	  pop	  edx
	  pop	  eax
	  ret


;-----------------------------;
;  enable an IRQ.  CL = irq   ;
;-----------------------------;
enable_irq:
	  push	  eax
	  push	  ecx

	  cmp	  cl, 8
	  jb	  .master

	  sub	  cl, 8
	  mov	  ah, 1
	  shl	  ah, cl
	  xor	  ah, 0xFF

	  in	  al, 0xA1
	  and	  al, ah
	  out	  0xA1, al

	  pop	  ecx
	  pop	  eax
	  ret
  .master:
	  mov	  ah, 1
	  shl	  ah, cl
	  xor	  ah, 0xFF

	  in	  al, 0x21
	  and	  al, ah
	  out	  0x21, al

	  pop	  ecx
	  pop	  eax
	  ret


;------------------------------;
;  disable an IRQ.  CL = irq   ;
;------------------------------;
disable_irq:
	  push	  eax
	  push	  ecx

	  cmp	  cl, 8
	  jb	  .master

	  sub	  cl, 8
	  mov	  ah, 1
	  shl	  ah, cl

	  in	  al, 0xA1
	  or	  al, ah
	  out	  0xA1, al

	  pop	  ecx
	  pop	  eax
	  ret
  .master:
	  mov	  ah, 1
	  shl	  ah, cl

	  in	  al, 0x21
	  or	  al, ah
	  out	  0x21, al

	  pop	  ecx
	  pop	  eax
	  ret