;----------------------------------------------------------;
; BOS 0.03                       Christoffer Bubach, 2004. ;
;----------------------------------------------------------;
;                                                          ;
;    Textmode screen functions.                            ;
;                                                          ;
;----------------------------------------------------------;

  ;-------;
  ; TODO: ;
  ;       ;------------------------------------------------;
  ; - Have 80*25 values in vars so that they can           ;
  ;   be changed when higher text res. is supported.       ;
  ;                                                        ;
  ; - Make all functions have (e)ax as standard I/O reg?   ;
  ;--------------------------------------------------------;


;--------------------------------------;
;  change cursor attribs               ;
;            in: cx = cursor attribs   ;
;--------------------------------------;
changecursor:
	  push	  ax
	  push	  dx

	  mov	  dx, 0x3D4
	  mov	  al, 0x0A
	  mov	  ah, ch
	  out	  dx, ax
	  inc	  ax
	  mov	  ah, cl
	  out	  dx, ax

	  pop	  dx
	  pop	  ax
	  ret


;--------------------;
; Clear the screen.  ;
;--------------------;
cls:
	  push	  dx
	  push	  cx

	  mov	  cx, 25
     .loop:
	  call	  scroll_up
	  loop	  .loop

	  mov	  dh, 0
	  mov	  dl, 0
	  call	  setcursorxy

	  pop	  cx
	  pop	  dx
	  ret


;-----------------------------------;
;  get the cursor pos               ;
;            out: dh = x   dl = y   ;
;-----------------------------------;
getcursorxy:
	  push	  ax
	  push	  bx

	  call	  getcursor
	  mov	  ax, bx
	  mov	  dl, 80
	  div	  dl
	  mov	  dl, al
	  mov	  dh, ah

	  pop	  bx
	  pop	  ax
	  ret


;-------------------------------;
;  get the cursor pos           ;
;            out: bx = offset   ;
;-------------------------------;
getcursor:
	  push	  ax
	  push	  dx

	  mov	  dx, 0x3D4
	  mov	  al, 0x0E
	  out	  dx, al
	  inc	  dx
	  in	  al, dx
	  mov	  bh, al
	  mov	  al, 0x0F
	  dec	  dx
	  out	  dx, al
	  inc	  dx
	  in	  al, dx
	  mov	  bl, al

	  pop	  dx
	  pop	  ax
	  ret


;------------------------------;
;  set the cursor to:          ;
;            dh = x   dl = y   ;
;------------------------------;
setcursorxy:
	  push	  ax
	  push	  bx
	  push	  dx

	  movzx   bx, dl
	  mov	  ax, bx
	  shl	  ax, 4 			      ; dont forget to change this
	  shl	  bx, 6 			      ; when higher text res. is supported
	  add	  bx, ax
	  movzx   ax, dh
	  add	  bx, ax

	  call	  setcursor

	  pop	  dx
	  pop	  bx
	  pop	  ax
	  ret


;--------------------------;
;  set the cursor to:      ;
;            bx = offset   ;
;--------------------------;
setcursor:
	  push	  ax
	  push	  bx
	  push	  dx

	  mov	  al, 0x0E
	  mov	  ah, bh
	  mov	  dx, 0x3D4
	  out	  dx, ax
	  inc	  ax
	  mov	  ah, bl
	  out	  dx, ax

	  pop	  dx
	  pop	  bx
	  pop	  ax
	  ret


;-----------------------;
;  cursor position +1   ;
;-----------------------;
inccursor:
	  push	  bx

	  call	  getcursor
	  cmp	  bx, 0x7CF
	  jne	  .cont
	  call	  scroll_up
	  jmp	  .end

     .cont:
	  inc	  bx
	  call	  setcursor

     .end:
	  pop	  bx
	  ret


;-----------------------;
;  cursor position -1   ;
;-----------------------;
deccursor:
	  push	  bx

	  call	  getcursor
	  cmp	  bx, 0
	  je	  .end

	  dec	  bx
	  call	  setcursor

     .end:
	  pop	  bx
	  ret



;---------------------------------------;
;  backspace - delete last typed char   ;
;---------------------------------------;
backspace:
	  push	  bx

	  call	  deccursor

	  mov	  bh, 0x07
	  mov	  bl, byte ' '
	  call	  print_char

	  call	  deccursor

	  pop	  bx
	  ret



;-----------------------------------------;
;  print char                             ;
;            in: bl = char, bh = attrib   ;
;-----------------------------------------;
print_char:
	  push	  eax
	  push	  bx

	  cmp	  bl, 13
	  jne	  .cont
	  call	  new_line
	  jmp	  .done

     .cont:
	  cmp	  bl, 10			      ; ignore
	  je	  .done

	  call	  getcursor

	  movzx   eax, bx
	  pop	  bx
	  mov	  [es:(eax*2 + 0xB8000)], bx

	  call	  inccursor
	  push	  bx

     .done:
	  pop	  bx
	  pop	  eax
	  ret



;------------------------------------------;
;  print 32-bit hex value                  ;
;            in: eax = value, bh = color   ;
;------------------------------------------;
	  hex2ascii db	 "0123456789ABCDEF"

print_hex32:
	  push	  eax
	  push	  bx
	  push	  cx
	  push	  edx
	  push	  esi

	  mov	  cx, 8
	  lea	  esi, [hex2ascii]
.print_it:
	  rol	  eax, 4
	  movzx   edx, al
	  and	  dl, 0x0F
	  mov	  bl, [esi+edx]
	  call	  print_char
	  loop	  .print_it

	  pop	  esi
	  pop	  edx
	  pop	  cx
	  pop	  bx
	  pop	  eax
	  ret



;--------------------------------------------;
;  display a asciiz message on the screen    ;
;            in: esi = message, al = color   ;
;--------------------------------------------;
print:
	  push	  ax
	  push	  bx

	  mov	  bh, al

     .displaychar:
	  lodsb
	  or	  al, al
	  jz	  .done

	  mov	  bl, al
	  call	  print_char
	  jmp	  .displaychar

     .done:
	  pop	  bx
	  pop	  ax
	  ret



;-----------------------------;
;  make a new line (CR, LF)   ;
;-----------------------------;
new_line:
	  push	  dx

	  call	  getcursorxy
	  cmp	  dl, 24
	  jb	  .newline

	  call	  scroll_up
	  jmp	  .done

     .newline:
	  call	  getcursorxy
	  mov	  dh, 0
	  inc	  dl
	  call	  setcursorxy

     .done:
	  pop	  dx
	  ret


;----------------;
;  scrolling..   ;
;----------------;
scroll_up:
	  pushad
	  call	  _scroll_up
	  popad

     .mv_curs:
	  mov	  bx, 80*24
	  call	  setcursor
	  ret

     _scroll_up:
	  push	  edi
	  push	  esi

	  mov	  edi, 0xB8000
	  mov	  esi, 0xB8000+160
	  mov	  ecx, (160*24)/4
	  rep	  movsd

	  mov	  edi, 0xB8000+160*24
	  mov	  ecx, 80/2
	  mov	  eax, 0x07000700
	  rep	  stosd

	  pop	  esi
	  pop	  edi
	  ret

